# Auto-generated by gen_format_sets.py
from pyegeria._output_format_models import Column, Format, ActionParameter, FormatSet, FormatSetDict

generated_format_sets = FormatSetDict({
    'Agreement-DrE': FormatSet(target_type='Agreement', heading='Agreement-DrE Attributes', description='Auto-generated format for Agreement (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Identifier', key='agreement_identifier'), Column(name='Category', key='category'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Version Identifier', key='version_identifier'), Column(name='Agreement Actors', key='agreement_actors'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Business-Imperative-DrE': FormatSet(target_type='Business Imperative', heading='Business-Imperative-DrE Attributes', description='Auto-generated format for Business Imperative (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Category-DrE': FormatSet(target_type='Category', heading='Category-DrE Attributes', description='Auto-generated format for Category (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Description', key='description'), Column(name='In Glossary', key='in_glossary'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'certification_type-DrE': FormatSet(target_type='certification_type', heading='certification_type-DrE Attributes', description='Auto-generated format for certification_type (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Document Identifier', key='document_identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Details', key='details'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Collection-DrE': FormatSet(target_type='Collection', heading='Collection-DrE Attributes', description='Auto-generated format for Collection (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Classifications', key='classifications'), Column(name='Category', key='category'), Column(name='Version Identifier', key='current_version'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Data-Class-DrE': FormatSet(target_type='Data Class', heading='Data-Class-DrE Attributes', description='Auto-generated format for Data Class (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Status', key='element_status'), Column(name='Namespace', key='namespace'), Column(name='Match Property Names', key='match_property_names'), Column(name='Match Threshold', key='match_threshold'), Column(name='IsCaseSensitive', key='is_case_sensitive'), Column(name='Data Type', key='data_type'), Column(name='Allow Duplicate Values', key='duplicates_allowed'), Column(name='isNullable', key='is_nullable'), Column(name='Default Value', key='default_value'), Column(name='Average Value', key='avg_value'), Column(name='Value List', key='value_list'), Column(name='Value Range From', key='value_range_from'), Column(name='Value Range To', key='value_range_to'), Column(name='Sample Values', key='sample_values'), Column(name='Data Patterns', key='data_patterns'), Column(name='In Data Dictionary', key='in_data_dictionary'), Column(name='Containing Data Class', key='containing_data_class'), Column(name='Specializes Data Class', key='specializes_data_class'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Data-Dictionary-DrE': FormatSet(target_type='Data Dictionary', heading='Data-Dictionary-DrE Attributes', description='Auto-generated format for Data Dictionary (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Data-Field-DrE': FormatSet(target_type='Data Field', heading='Data-Field-DrE Attributes', description='Auto-generated format for Data Field (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Status', key='element_status'), Column(name='Data Type', key='data_type'), Column(name='Position', key='position'), Column(name='Minimum Cardinality', key='min_cardinality'), Column(name='Maximum Cardinality', key='max_cardinality'), Column(name='In Data Structure', key='in_data_structure'), Column(name='Data Class', key='data_class'), Column(name='Glossary Term', key='glossary_term'), Column(name='isNullable', key='is_nullable'), Column(name='Minimum Length', key='min_length'), Column(name='Length', key='length'), Column(name='Precision', key='precision'), Column(name='Ordered Values', key='ordered_values'), Column(name='Units', key='units'), Column(name='Default Value', key='default_value'), Column(name='Version Identifier', key='version_id'), Column(name='In Data Dictionary', key='in_data_dictionary'), Column(name='Parent Data Field', key='parent_data_field'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Data-Processing-Purpose-DrE': FormatSet(target_type='Data Processing Purpose', heading='Data-Processing-Purpose-DrE Attributes', description='Auto-generated format for Data Processing Purpose (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Data-Sharing-Agreement-DrE': FormatSet(target_type='Data Sharing Agreement', heading='Data-Sharing-Agreement-DrE Attributes', description='Auto-generated format for Data Sharing Agreement (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Identifier', key='identifier'), Column(name='Status', key='element_status'), Column(name='User_Defined_Status', key='user_defined_status'), Column(name='Version Identifier', key='version_identifier'), Column(name='Product Manager', key='product_manager'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Data-Specification-DrE': FormatSet(target_type='Data Specification', heading='Data-Specification-DrE Attributes', description='Auto-generated format for Data Specification (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Collection Type', key='collection_type'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Data-Structure-DrE': FormatSet(target_type='Data Structure', heading='Data-Structure-DrE Attributes', description='Auto-generated format for Data Structure (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Status', key='element_status'), Column(name='In Data Specification', key='in_data_spec'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Digital-Product-Catalog-DrE': FormatSet(target_type='Digital Product Catalog', heading='Digital-Product-Catalog-DrE Attributes', description='Auto-generated format for Digital Product Catalog (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Classifications', key='classifications'), Column(name='Category', key='category'), Column(name='Version Identifier', key='current_version'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Digital-Product-DrE': FormatSet(target_type='Digital Product', heading='Digital-Product-DrE Attributes', description='Auto-generated format for Digital Product (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Product Name', key='product_name'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Category', key='category'), Column(name='Identifier', key='identifier'), Column(name='Maturity', key='maturity'), Column(name='Service Life', key='service_life'), Column(name='Introduction Date', key='introduction_date'), Column(name='Next Version Date', key='next_version_date'), Column(name='Withdrawal Date', key='withdrawal_date'), Column(name='Version Identifier', key='current_version'), Column(name='Product Manager', key='product_manager'), Column(name='Agreements', key='agreements'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Digital-Subscription-DrE': FormatSet(target_type='Digital Subscription', heading='Digital-Subscription-DrE Attributes', description='Auto-generated format for Digital Subscription (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Identifier', key='identifier'), Column(name='Category', key='category'), Column(name='Status', key='element_status'), Column(name='User_Defined_Status', key='user_defined_status'), Column(name='Support Level', key='support_level'), Column(name='Service Levels', key='service_levels'), Column(name='Version Identifier', key='version_identifier'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Glossary-DrE': FormatSet(target_type='Glossary', heading='Glossary-DrE Attributes', description='Auto-generated format for Glossary (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Language', key='language'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Governance-Action-DrE': FormatSet(target_type='Governance Action', heading='Governance-Action-DrE Attributes', description='Auto-generated format for Governance Action (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Governance-Approach-DrE': FormatSet(target_type='Governance Approach', heading='Governance-Approach-DrE Attributes', description='Auto-generated format for Governance Approach (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Governance-Control-DrE': FormatSet(target_type='Governance Control', heading='Governance-Control-DrE Attributes', description='Auto-generated format for Governance Control (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='measurement', key='measurement'), Column(name='target', key='target'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Governance-Driver-DrE': FormatSet(target_type='Governance Driver', heading='Governance-Driver-DrE Attributes', description='Auto-generated format for Governance Driver (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Governance-Obligation-DrE': FormatSet(target_type='Governance Obligation', heading='Governance-Obligation-DrE Attributes', description='Auto-generated format for Governance Obligation (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Governance-Policy-DrE': FormatSet(target_type='Governance Policy', heading='Governance-Policy-DrE Attributes', description='Auto-generated format for Governance Policy (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Governance-Principle-DrE': FormatSet(target_type='Governance Principle', heading='Governance-Principle-DrE Attributes', description='Auto-generated format for Governance Principle (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'governance_procedure-DrE': FormatSet(target_type='governance_procedure', heading='governance_procedure-DrE Attributes', description='Auto-generated format for governance_procedure (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Document Identifier', key='document_identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implementation Description', key='implementation_description'), Column(name='Supports Policies', key='supports_policies'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'governance_responsibility-DrE': FormatSet(target_type='governance_responsibility', heading='governance_responsibility-DrE Attributes', description='Auto-generated format for governance_responsibility (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Document Identifier', key='document_identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implementation Description', key='implementation_description'), Column(name='Supports Policies', key='supports_policies'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'governance_rule-DrE': FormatSet(target_type='governance_rule', heading='governance_rule-DrE Attributes', description='Auto-generated format for governance_rule (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'governance_strategy-DrE': FormatSet(target_type='governance_strategy', heading='governance_strategy-DrE Attributes', description='Auto-generated format for governance_strategy (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Information-Supply-Chain-DrE': FormatSet(target_type='Information Supply Chain', heading='Information-Supply-Chain-DrE Attributes', description='Auto-generated format for Information Supply Chain (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Scope', key='scope'), Column(name='Purposes', key='purposes'), Column(name='Nested Information Supply Chains', key='nested_info_supply_chains'), Column(name='In Information Supply Chain', key='in_supply_chain'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid'), Column(name='Merge Update', key='merge_update')])]),
    'license_type-DrE': FormatSet(target_type='license_type', heading='license_type-DrE Attributes', description='Auto-generated format for license_type (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Document Identifier', key='document_identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Details', key='details'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'naming_standard_rule-DrE': FormatSet(target_type='naming_standard_rule', heading='naming_standard_rule-DrE Attributes', description='Auto-generated format for naming_standard_rule (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Document Identifier', key='document_identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implementation Description', key='implementation_description'), Column(name='Supports Policies', key='supports_policies'), Column(name='Name Patterns', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Project-DrE': FormatSet(target_type='Project', heading='Project-DrE Attributes', description='Auto-generated format for Project (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Project Type', key='project_type'), Column(name='Category', key='category'), Column(name='Identifier', key='project_identifier'), Column(name='Mission', key='mission'), Column(name='Purposes', key='purposes'), Column(name='Start Date', key='start_date'), Column(name='Planned End Date', key='end_date'), Column(name='Priority', key='priority'), Column(name='Project Phase', key='project_phase'), Column(name='Project Status', key='project_status'), Column(name='Project Health', key='project_health'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Regulation-Article-DrE': FormatSet(target_type='Regulation Article', heading='Regulation-Article-DrE Attributes', description='Auto-generated format for Regulation Article (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'regulation-DrE': FormatSet(target_type='regulation', heading='regulation-DrE Attributes', description='Auto-generated format for regulation (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Regulation Source', key='regulation_source'), Column(name='Regulators', key='regulators'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'security_access_control-DrE': FormatSet(target_type='security_access_control', heading='security_access_control-DrE Attributes', description='Auto-generated format for security_access_control (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='criteria', key='criteria'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'security_group-DrE': FormatSet(target_type='security_group', heading='security_group-DrE Attributes', description='Auto-generated format for security_group (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'service_level_objective-DrE': FormatSet(target_type='service_level_objective', heading='service_level_objective-DrE Attributes', description='Auto-generated format for service_level_objective (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Solution-Blueprint-DrE': FormatSet(target_type='Solution Blueprint', heading='Solution-Blueprint-DrE Attributes', description='Auto-generated format for Solution Blueprint (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Status', key='element_status'), Column(name='Version Identifier', key='version_id'), Column(name='Solution Components', key='solution_components'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Solution-Component-DrE': FormatSet(target_type='Solution Component', heading='Solution-Component-DrE Attributes', description='Auto-generated format for Solution Component (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Qualified Name', key='qualified_name'), Column(name='Category', key='category'), Column(name='Description', key='description'), Column(name='Status', key='element_status'), Column(name='Solution Component Type', key='soln_comp_type'), Column(name='Planned Deployed Implementation Type', key='planned_deployed_impl_type'), Column(name='Initial Status', key='initial_status'), Column(name='In Solution Components', key='in_components'), Column(name='In Solution Blueprints', key='solution_blueprints'), Column(name='In Information Supply Chains', key='in_supply_chains'), Column(name='Actors', key='actors'), Column(name='GUID', key='guid'), Column(name='Merge Update', key='merge_update')])]),
    'Solution-Role-DrE': FormatSet(target_type='Solution Role', heading='Solution-Role-DrE Attributes', description='Auto-generated format for Solution Role (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Name', key='name'), Column(name='Description', key='description'), Column(name='Title', key='title'), Column(name='Scope', key='scope'), Column(name='identifier', key='identifier'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Category', key='category'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])]),
    'Threat-Definition-DrE': FormatSet(target_type='Threat Definition', heading='Threat-Definition-DrE Attributes', description='Auto-generated format for Threat Definition (Create).', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Category', key='category'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Version Identifier', key='version_identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Status', key='element_status'), Column(name='User Defined Status', key='user_defined_status'), Column(name='Qualified Name', key='qualified_name'), Column(name='GUID', key='guid')])])
})
