# Auto-generated by gen_report_specs.py
from pyegeria._output_format_models import Column, Format, ActionParameter, FormatSet, FormatSetDict

generated_format_sets = FormatSetDict({
    'Agreement-DrE': FormatSet(target_type='Agreement', heading='Agreement-DrE Attributes', description='Auto-generated format for Agreement (Create).', family='Digital Product Manager', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Identifier', key='agreement_identifier'), Column(name='Agreement Actors', key='agreement_actors')])], action=ActionParameter(function='CollectionManager.find_collections', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['Agreement']})),
    'Business-Imperative-DrE': FormatSet(target_type='Business Imperative', heading='Business-Imperative-DrE Attributes', description='Auto-generated format for Business Imperative (Create).', family='Governance Officer', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results')])], action=ActionParameter(function='GovernanceOfficer.find_governance_definitions', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['GovernanceDriver']})),
    'Campaign-DrE': FormatSet(target_type='Campaign', heading='Campaign-DrE Attributes', description='Auto-generated format for Campaign (Create).', family='Project', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Project Type', key='project_type'), Column(name='Identifier', key='project_identifier'), Column(name='Mission', key='mission'), Column(name='Purposes', key='purposes'), Column(name='Start Date', key='start_date'), Column(name='Planned End Date', key='end_date'), Column(name='Priority', key='priority'), Column(name='Project Phase', key='project_phase'), Column(name='Project Status', key='project_status'), Column(name='Project Health', key='project_health')])], action=ActionParameter(function='ProjectManager.find_projects', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'classification_names': ['Campaign']})),
    'Certification-Type-DrE': FormatSet(target_type='Certification Type', heading='Certification-Type-DrE Attributes', description='Auto-generated format for Certification Type (Create).', family='Governance Officer', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Document Identifier', key='document_identifier'), Column(name='Scope', key='scope'), Column(name='Usage', key='usage'), Column(name='Importance', key='importance'), Column(name='Details', key='details'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Restrictions', key='restrictions'), Column(name='Obligations', key='obligations'), Column(name='Entitlements', key='Entitlements')])], action=ActionParameter(function='GovernanceOfficer.find_governance_definitions', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['CertificationType']})),
    'Collection-DrE': FormatSet(target_type='Collection', heading='Collection-DrE Attributes', description='Auto-generated format for Collection (Create).', family='Digital Product Manager', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description')])], action=ActionParameter(function='CollectionManager.find_collections', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'])),
    'Comment-DrE': FormatSet(target_type='Comment', heading='Comment-DrE Attributes', description='Auto-generated format for Comment (Create).', family='Feedback Manager', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Associated Element', key='associated_element'), Column(name='Comment Text', key='comment_text'), Column(name='Comment Type', key='comment_type')])], action=ActionParameter(function='GovernanceOfficer.find_governance_definitions', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'])),
    'CSV-File-DrE': FormatSet(target_type='CSV File', heading='CSV-File-DrE Attributes', description='Auto-generated format for CSV File (Create).', family='Digital Product Manager', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='File Path', key='file_path'), Column(name='File Encoding', key='file_encoding'), Column(name='File Extension', key='file_extension'), Column(name='File System Name', key='file_system_name')])]),
    'Data-Class-DrE': FormatSet(target_type='Data Class', heading='Data-Class-DrE Attributes', description='Auto-generated format for Data Class (Create).', family='Data Designer', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Namespace', key='namespace'), Column(name='Match Property Names', key='match_property_names'), Column(name='Match Threshold', key='match_threshold'), Column(name='IsCaseSensitive', key='is_case_sensitive'), Column(name='Data Type', key='data_type'), Column(name='Allow Duplicate Values', key='duplicates_allowed'), Column(name='isNullable', key='is_nullable'), Column(name='Default Value', key='default_value'), Column(name='Average Value', key='avg_value'), Column(name='Value List', key='value_list'), Column(name='Value Range From', key='value_range_from'), Column(name='Value Range To', key='value_range_to'), Column(name='Sample Values', key='sample_values'), Column(name='Data Patterns', key='data_patterns'), Column(name='In Data Dictionary', key='in_data_dictionary'), Column(name='Containing Data Class', key='containing_data_class'), Column(name='Specializes Data Class', key='specializes_data_class')])], action=ActionParameter(function='DataDesigner.find_data_classes', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'])),
    'Data-Dictionary-DrE': FormatSet(target_type='Data Dictionary', heading='Data-Dictionary-DrE Attributes', description='Auto-generated format for Data Dictionary (Create).', family='Data Designer', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description')])], action=ActionParameter(function='CollectionManager.find_collections', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['DataDictionary']})),
    'Data-Field-DrE': FormatSet(target_type='Data Field', heading='Data-Field-DrE Attributes', description='Auto-generated format for Data Field (Create).', family='Data Designer', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Data Type', key='data_type'), Column(name='Position', key='position'), Column(name='Minimum Cardinality', key='min_cardinality'), Column(name='Maximum Cardinality', key='max_cardinality'), Column(name='In Data Structure', key='in_data_structure'), Column(name='In Data Dictionary', key='in_data_dictionary'), Column(name='Data Class', key='data_class'), Column(name='isNullable', key='is_nullable'), Column(name='Minimum Length', key='min_length'), Column(name='Length', key='length'), Column(name='Precision', key='precision'), Column(name='Ordered Values', key='ordered_values'), Column(name='Units', key='units'), Column(name='Default Value', key='default_value')])], action=ActionParameter(function='DataDesigner.find_data_fields', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'])),
    'Data-Processing-Purpose-DrE': FormatSet(target_type='Data Processing Purpose', heading='Data-Processing-Purpose-DrE Attributes', description='Auto-generated format for Data Processing Purpose (Create).', family='Governance Officer', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results')])], action=ActionParameter(function='GovernanceOfficer.find_governance_definitions', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['DataProcessingPurpose']})),
    'Data-Sharing-Agreement-DrE': FormatSet(target_type='Data Sharing Agreement', heading='Data-Sharing-Agreement-DrE Attributes', description='Auto-generated format for Data Sharing Agreement (Create).', family='Digital Product Manager', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Identifier', key='identifier'), Column(name='Product Manager', key='product_manager')])], action=ActionParameter(function='CollectionManager.find_collections', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'classification_names': ['DataSharingAgreement']})),
    'Data-Specification-DrE': FormatSet(target_type='Data Specification', heading='Data-Specification-DrE Attributes', description='Auto-generated format for Data Specification (Create).', family='Data Designer', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description')])], action=ActionParameter(function='CollectionManager.find_collections', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['DataSpec']})),
    'Data-Structure-DrE': FormatSet(target_type='Data Structure', heading='Data-Structure-DrE Attributes', description='Auto-generated format for Data Structure (Create).', family='Data Designer', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='In Data Specification', key='in_data_spec'), Column(name='In Data Dictionary', key='in_data_dictionary')])], action=ActionParameter(function='DataDesigner.find_data_structures', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'])),
    'Digital-Product-Catalog-DrE': FormatSet(target_type='Digital Product Catalog', heading='Digital-Product-Catalog-DrE Attributes', description='Auto-generated format for Digital Product Catalog (Create).', family='Digital Product Manager', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description')])], action=ActionParameter(function='CollectionManager.find_collections', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['DigitalProductCatalog']})),
    'Digital-Product-DrE': FormatSet(target_type='Digital Product', heading='Digital-Product-DrE Attributes', description='Auto-generated format for Digital Product (Create).', family='Digital Product Manager', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Product Name', key='product_name'), Column(name='Identifier', key='identifier'), Column(name='Maturity', key='maturity'), Column(name='Service Life', key='service_life'), Column(name='Introduction Date', key='introduction_date'), Column(name='Next Version Date', key='next_version_date'), Column(name='Withdrawal Date', key='withdrawal_date'), Column(name='Product Manager', key='product_manager'), Column(name='Agreements', key='agreements')])], action=ActionParameter(function='CollectionManager.find_collections', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['DigitalProduct']})),
    'Digital-Subscription-DrE': FormatSet(target_type='Digital Subscription', heading='Digital-Subscription-DrE Attributes', description='Auto-generated format for Digital Subscription (Create).', family='Digital Product Manager', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Identifier', key='identifier'), Column(name='Support Level', key='support_level'), Column(name='Service Levels', key='service_levels')])], action=ActionParameter(function='CollectionManager.find_collections', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['DigitalSubscription']})),
    'External-Data-Source-DrE': FormatSet(target_type='External Data Source', heading='External-Data-Source-DrE Attributes', description='Auto-generated format for External Data Source (Create).', family='External Reference', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Reference Title', key='reference_title'), Column(name='Reference Abstract', key='reference_abstract'), Column(name='Authors', key='authors'), Column(name='Organization', key='organization'), Column(name='Sources', key='reference_sources'), Column(name='License', key='license'), Column(name='Copyright', key='copyright'), Column(name='Attribution', key='category')])], action=ActionParameter(function='ExternalReference.find_external_references', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['ExternalDataSource']})),
    'External-Reference-DrE': FormatSet(target_type='External Reference', heading='External-Reference-DrE Attributes', description='Auto-generated format for External Reference (Create).', family='External Reference', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Reference Title', key='reference_title'), Column(name='Reference Abstract', key='reference_abstract'), Column(name='Authors', key='authors'), Column(name='Organization', key='organization'), Column(name='Sources', key='reference_sources'), Column(name='License', key='license'), Column(name='Copyright', key='copyright'), Column(name='Attribution', key='category'), Column(name='Number of Pages', key='number_of_pages'), Column(name='Page Range', key='page_range'), Column(name='Publication Series', key='publication_series'), Column(name='Publication Series Volume', key='pub_series_volume'), Column(name='Publisher', key='publisher'), Column(name='First Publication Date', key='first_pub_date'), Column(name='Publication Date', key='pub_date'), Column(name='Publication City', key='pub_city'), Column(name='Publication Year', key='pub_year'), Column(name='Publication Numbers', key='pub_numbers')])], action=ActionParameter(function='ExternalReference.find_external_references', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['CitedDocument']})),
    'Glossary-DrE': FormatSet(target_type='Glossary', heading='Glossary-DrE Attributes', description='Auto-generated format for Glossary (Create).', family='Glossary', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Language', key='language')])], action=ActionParameter(function='CollectionManager.find_collections', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['Glossary']})),
    'Glossary-Term-DrE': FormatSet(target_type='Glossary Term', heading='Glossary-Term-DrE Attributes', description='Auto-generated format for Glossary Term (Create).', family='Glossary', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='term_name'), Column(name='Glossary', key='glossary_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Folders', key='folders'), Column(name='Abbreviation', key='abbreviation'), Column(name='Example', key='example'), Column(name='Usage', key='usage')])], action=ActionParameter(function='GlossaryManager.find_glossary_terms', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'])),
    'Governance-Action-DrE': FormatSet(target_type='Governance Action', heading='Governance-Action-DrE Attributes', description='Auto-generated format for Governance Action (Create).', family='Governance Officer', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results')])], action=ActionParameter(function='GovernanceOfficer.find_governance_definitions', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['GovernanceAction']})),
    'Governance-Approach-DrE': FormatSet(target_type='Governance Approach', heading='Governance-Approach-DrE Attributes', description='Auto-generated format for Governance Approach (Create).', family='Governance Officer', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results')])], action=ActionParameter(function='GovernanceOfficer.find_governance_definitions', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['GovernanceApproach']})),
    'Governance-Control-DrE': FormatSet(target_type='Governance Control', heading='Governance-Control-DrE Attributes', description='Auto-generated format for Governance Control (Create).', family='Governance Officer', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='measurement', key='measurement'), Column(name='target', key='target'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results')])], action=ActionParameter(function='GovernanceOfficer.find_governance_definitions', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['GovernanceMetric']})),
    'Governance-Driver-DrE': FormatSet(target_type='Governance Driver', heading='Governance-Driver-DrE Attributes', description='Auto-generated format for Governance Driver (Create).', family='Governance Officer', formats=[Format(types=['ALL'], columns=[Column(name='Usage', key='usage'), Column(name='Summary', key='summary'), Column(name='Scope', key='scope'), Column(name='Results', key='results'), Column(name='Outcomes', key='outcomes'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Identifier', key='identifier')])], action=ActionParameter(function='GovernanceOfficer.find_governance_definitions', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['GovernanceDriver']})),
    'Governance-Obligation-DrE': FormatSet(target_type='Governance Obligation', heading='Governance-Obligation-DrE Attributes', description='Auto-generated format for Governance Obligation (Create).', family='Governance Officer', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results')])], action=ActionParameter(function='GovernanceOfficer.find_governance_definitions', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['GovernanceObligation']})),
    'Governance-Policy-DrE': FormatSet(target_type='Governance Policy', heading='Governance-Policy-DrE Attributes', description='Auto-generated format for Governance Policy (Create).', family='Governance Officer', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results')])], action=ActionParameter(function='GovernanceOfficer.find_governance_definitions', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['GovernancePolicy']})),
    'Governance-Principle-DrE': FormatSet(target_type='Governance Principle', heading='Governance-Principle-DrE Attributes', description='Auto-generated format for Governance Principle (Create).', family='Governance Officer', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results')])], action=ActionParameter(function='GovernanceOfficer.find_governance_definitions', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['GovernancePrinciple']})),
    'Governance-Procedure-DrE': FormatSet(target_type='Governance Procedure', heading='Governance-Procedure-DrE Attributes', description='Auto-generated format for Governance Procedure (Create).', family='Governance Officer', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Document Identifier', key='document_identifier'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implementation Description', key='implementation_description'), Column(name='Supports Policies', key='supports_policies'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results')])], action=ActionParameter(function='GovernanceOfficer.find_governance_definitions', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['GovernanceProcedure']})),
    'Governance-Responsibility-DrE': FormatSet(target_type='Governance Responsibility', heading='Governance-Responsibility-DrE Attributes', description='Auto-generated format for Governance Responsibility (Create).', family='Governance Officer', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Document Identifier', key='document_identifier'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implementation Description', key='implementation_description'), Column(name='Supports Policies', key='supports_policies'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results')])], action=ActionParameter(function='GovernanceOfficer.find_governance_definitions', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['GovernanceResponsibility']})),
    'Governance-Rule-DrE': FormatSet(target_type='Governance Rule', heading='Governance-Rule-DrE Attributes', description='Auto-generated format for Governance Rule (Create).', family='Governance Officer', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Document Identifier', key='document_identifier'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implementation Description', key='implementation_description'), Column(name='Supports Policies', key='supports_policies'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results')])], action=ActionParameter(function='GovernanceOfficer.find_governance_definitions', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['GovernanceRule']})),
    'Governance-Strategy-DrE': FormatSet(target_type='Governance Strategy', heading='Governance-Strategy-DrE Attributes', description='Auto-generated format for Governance Strategy (Create).', family='Governance Officer', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results')])], action=ActionParameter(function='GovernanceOfficer.find_governance_definitions', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['GovernanceStrategy']})),
    'Informal-Tag-DrE': FormatSet(target_type='Informal Tag', heading='Informal-Tag-DrE Attributes', description='Auto-generated format for Informal Tag (Create).', family='Feedback Manager', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description')])], action=ActionParameter(function='Client2.find_tags', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'])),
    'Information-Supply-Chain-DrE': FormatSet(target_type='Information Supply Chain', heading='Information-Supply-Chain-DrE Attributes', description='Auto-generated format for Information Supply Chain (Create).', family='Solution Architect', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Scope', key='scope'), Column(name='Purposes', key='purposes'), Column(name='Nested Information Supply Chains', key='nested_info_supply_chains'), Column(name='In Information Supply Chain', key='in_supply_chain')])], action=ActionParameter(function='SolutionArchitect.find_information_supply_chains', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'])),
    'License-Type-DrE': FormatSet(target_type='License Type', heading='License-Type-DrE Attributes', description='Auto-generated format for License Type (Create).', family='Governance Officer', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Document Identifier', key='document_identifier'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Summary', key='summary'), Column(name='Scope', key='scope'), Column(name='Usage', key='usage'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results'), Column(name='Description', key='description'), Column(name='Details', key='details'), Column(name='Restrictions', key='restrictions'), Column(name='Obligations', key='obligations'), Column(name='Entitlements', key='Entitlements')])], action=ActionParameter(function='GovernanceOfficer.find_governance_definitions', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'])),
    'Naming-Standard-Rule-DrE': FormatSet(target_type='Naming Standard Rule', heading='Naming-Standard-Rule-DrE Attributes', description='Auto-generated format for Naming Standard Rule (Create).', family='Governance Officer', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Document Identifier', key='document_identifier'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implementation Description', key='implementation_description'), Column(name='Supports Policies', key='supports_policies'), Column(name='Name Patterns', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results')])], action=ActionParameter(function='GovernanceOfficer.find_governance_definitions', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['NamingStandardRule']})),
    'NoteLog-DrE': FormatSet(target_type='NoteLog', heading='NoteLog-DrE Attributes', description='Auto-generated format for NoteLog (Create).', family='Feedback Manager', formats=[Format(types=['ALL'], columns=[Column(name='Journal Name', key='display_name'), Column(name='Associated Element', key='associated_element'), Column(name='Journal Description', key='journal_description'), Column(name='Note Entry', key='note_entry')])], action=ActionParameter(function='Client2.find_notes', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'])),
    'Personal-Project-DrE': FormatSet(target_type='Personal Project', heading='Personal-Project-DrE Attributes', description='Auto-generated format for Personal Project (Create).', family='Project', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Project Type', key='project_type'), Column(name='Identifier', key='project_identifier'), Column(name='Mission', key='mission'), Column(name='Purposes', key='purposes'), Column(name='Start Date', key='start_date'), Column(name='Planned End Date', key='end_date'), Column(name='Priority', key='priority'), Column(name='Project Phase', key='project_phase'), Column(name='Project Status', key='project_status'), Column(name='Project Health', key='project_health')])], action=ActionParameter(function='ProjectManager.find_projects', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'classification_names': ['PersonalProject']})),
    'Project-DrE': FormatSet(target_type='Project', heading='Project-DrE Attributes', description='Auto-generated format for Project (Create).', family='Project', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Project Type', key='project_type'), Column(name='Identifier', key='project_identifier'), Column(name='Mission', key='mission'), Column(name='Purposes', key='purposes'), Column(name='Start Date', key='start_date'), Column(name='Planned End Date', key='end_date'), Column(name='Priority', key='priority'), Column(name='Project Phase', key='project_phase'), Column(name='Project Status', key='project_status'), Column(name='Project Health', key='project_health')])], action=ActionParameter(function='ProjectManager.find_projects', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'])),
    'Regulation-Article-DrE': FormatSet(target_type='Regulation Article', heading='Regulation-Article-DrE Attributes', description='Auto-generated format for Regulation Article (Create).', family='Governance Officer', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results')])], action=ActionParameter(function='GovernanceOfficer.find_governance_definitions', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['RegulationArticle']})),
    'Regulation-DrE': FormatSet(target_type='Regulation', heading='Regulation-DrE Attributes', description='Auto-generated format for Regulation (Create).', family='Governance Officer', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Regulation Source', key='regulation_source'), Column(name='Regulators', key='regulators'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results')])], action=ActionParameter(function='GovernanceOfficer.find_governance_definitions', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['Regulation']})),
    'Security-Access-Control-DrE': FormatSet(target_type='Security Access Control', heading='Security-Access-Control-DrE Attributes', description='Auto-generated format for Security Access Control (Create).', family='Governance Officer', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results')])], action=ActionParameter(function='GovernanceOfficer.find_governance_definitions', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['SecurityAccessControl']})),
    'Security-Group-DrE': FormatSet(target_type='Security Group', heading='Security-Group-DrE Attributes', description='Auto-generated format for Security Group (Create).', family='Governance Officer', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results')])], action=ActionParameter(function='GovernanceOfficer.find_governance_definitions', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['SEcurityGroup']})),
    'security_access_control-DrE': FormatSet(target_type='security_access_control', heading='security_access_control-DrE Attributes', description='Auto-generated format for security_access_control (Create).', family='Governance Officer', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='criteria', key='criteria'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results')])], action=ActionParameter(function='GovernanceOfficer.find_governance_definitions', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['GovernanceZone']})),
    'Service-Level-Objectives-DrE': FormatSet(target_type='Service Level Objectives', heading='Service-Level-Objectives-DrE Attributes', description='Auto-generated format for Service Level Objectives (Create).', family='Governance Officer', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Document Identifier', key='document_identifier'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implementation Description', key='implementation_description'), Column(name='Supports Policies', key='supports_policies'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results')])], action=ActionParameter(function='GovernanceOfficer.find_governance_definitions', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['ServiceLevelObjectives']})),
    'Solution-Blueprint-DrE': FormatSet(target_type='Solution Blueprint', heading='Solution-Blueprint-DrE Attributes', description='Auto-generated format for Solution Blueprint (Create).', family='Solution Architect', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Solution Components', key='solution_components')])], action=ActionParameter(function='SolutionArchitect.find_solution_blueprints', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'])),
    'Solution-Component-DrE': FormatSet(target_type='Solution Component', heading='Solution-Component-DrE Attributes', description='Auto-generated format for Solution Component (Create).', family='Solution Architect', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Solution Component Type', key='soln_comp_type'), Column(name='Planned Deployed Implementation Type', key='planned_deployed_impl_type'), Column(name='In Solution Components', key='in_components'), Column(name='In Solution Blueprints', key='solution_blueprints'), Column(name='In Information Supply Chains', key='in_supply_chains'), Column(name='Actors', key='actors')])], action=ActionParameter(function='SolutionArchitect.find_solution_components', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'])),
    'Solution-Role-DrE': FormatSet(target_type='Solution Role', heading='Solution-Role-DrE Attributes', description='Auto-generated format for Solution Role (Create).', family='Solution Architect', formats=[Format(types=['ALL'], columns=[Column(name='Name', key='name'), Column(name='Description', key='description'), Column(name='Title', key='title'), Column(name='Scope', key='scope'), Column(name='identifier', key='identifier'), Column(name='Domain Identifier', key='domain_identifier')])], action=ActionParameter(function='SolutionArchitect.find_solution_roles', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'])),
    'Study-Project-DrE': FormatSet(target_type='Study Project', heading='Study-Project-DrE Attributes', description='Auto-generated format for Study Project (Create).', family='Project', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Project Type', key='project_type'), Column(name='Identifier', key='project_identifier'), Column(name='Mission', key='mission'), Column(name='Purposes', key='purposes'), Column(name='Start Date', key='start_date'), Column(name='Planned End Date', key='end_date'), Column(name='Priority', key='priority'), Column(name='Project Phase', key='project_phase'), Column(name='Project Status', key='project_status'), Column(name='Project Health', key='project_health')])], action=ActionParameter(function='ProjectManager.find_projects', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'classification_names': ['StudyProject']})),
    'Task-DrE': FormatSet(target_type='Task', heading='Task-DrE Attributes', description='Auto-generated format for Task (Create).', family='Project', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Description', key='description'), Column(name='Project Type', key='project_type'), Column(name='Identifier', key='project_identifier'), Column(name='Mission', key='mission'), Column(name='Purposes', key='purposes'), Column(name='Start Date', key='start_date'), Column(name='Planned End Date', key='end_date'), Column(name='Priority', key='priority'), Column(name='Project Phase', key='project_phase'), Column(name='Project Status', key='project_status'), Column(name='Project Health', key='project_health')])], action=ActionParameter(function='ProjectManager.find_projects', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'classification_names': ['Task']})),
    'Threat-Definition-DrE': FormatSet(target_type='Threat Definition', heading='Threat-Definition-DrE Attributes', description='Auto-generated format for Threat Definition (Create).', family='Governance Officer', formats=[Format(types=['ALL'], columns=[Column(name='Display Name', key='display_name'), Column(name='Summary', key='summary'), Column(name='Description', key='description'), Column(name='Domain Identifier', key='domain_identifier'), Column(name='Identifier', key='identifier'), Column(name='Usage', key='usage'), Column(name='Scope', key='scope'), Column(name='Importance', key='importance'), Column(name='Implications', key='implications'), Column(name='Outcomes', key='outcomes'), Column(name='Results', key='results')])], action=ActionParameter(function='GovernanceOfficer.find_governance_definitions', required_params=['search_string'], optional_params=['page_size', 'start_from', 'starts_with', 'ends_with', 'ignore_case'], spec_params={'metadata_element_types': ['ThreatDefinition']}))
})
